@@ -1,14 +1,14 @@
 import copy
+import itertools
 import multiprocessing
-import re
 import time
-import yaml
+import agate
 
 from contextlib import contextmanager
 
 import dbt.exceptions
 import dbt.flags
-import dbt.materializers
+import dbt.clients.agate_helper
 
 from dbt.contracts.connection import validate_connection
 from dbt.logger import GLOBAL_LOGGER as logger
@@ -24,9 +24,25 @@ class DefaultAdapter(object):
 
     requires = {}
 
-    @classmethod
-    def get_materializer(cls, model, existing):
-        return dbt.materializers.get_materializer(cls, model, existing)
+    context_functions = [
+        "already_exists",
+        "get_columns_in_table",
+        "get_missing_columns",
+        "query_for_existing",
+        "rename",
+        "drop",
+        "truncate",
+        "add_query",
+        "expand_target_column_types",
+        "quote_schema_and_table",
+        "execute"
+    ]
+
+    raw_functions = [
+        "get_status",
+        "get_result_from_cursor",
+        "quote",
+    ]
 
     ###
     # ADAPTER-SPECIFIC FUNCTIONS -- each of these must be overridden in
@@ -49,16 +65,6 @@ class DefaultAdapter(object):
         raise dbt.exceptions.NotImplementedException(
             '`date_function` is not implemented for this adapter!')
 
-    @classmethod
-    def dist_qualifier(cls):
-        raise dbt.exceptions.NotImplementedException(
-            '`dist_qualifier` is not implemented for this adapter!')
-
-    @classmethod
-    def sort_qualifier(cls):
-        raise dbt.exceptions.NotImplementedException(
-            '`sort_qualifier` is not implemented for this adapter!')
-
     @classmethod
     def get_status(cls, cursor):
         raise dbt.exceptions.NotImplementedException(
@@ -71,10 +77,15 @@ class DefaultAdapter(object):
             '`alter_column_type` is not implemented for this adapter!')
 
     @classmethod
-    def query_for_existing(cls, profile, schema, model_name=None):
+    def query_for_existing(cls, profile, schemas, model_name=None):
         raise dbt.exceptions.NotImplementedException(
             '`query_for_existing` is not implemented for this adapter!')
 
+    @classmethod
+    def get_existing_schemas(cls, profile, model_name=None):
+        raise dbt.exceptions.NotImplementedException(
+            '`get_existing_schemas` is not implemented for this adapter!')
+
     @classmethod
     def check_schema_exists(cls, profile, schema):
         raise dbt.exceptions.NotImplementedException(
@@ -85,58 +96,75 @@ class DefaultAdapter(object):
         raise dbt.exceptions.NotImplementedException(
             '`cancel_connection` is not implemented for this adapter!')
 
+    @classmethod
+    def create_csv_table(cls, profile, schema, table_name, agate_table):
+        raise dbt.exceptions.NotImplementedException(
+            '`create_csv_table` is not implemented for this adapter!')
+
+    @classmethod
+    def reset_csv_table(cls, profile, schema, table_name, agate_table,
+                        full_refresh=False):
+        raise dbt.exceptions.NotImplementedException(
+            '`reset_csv_table` is not implemented for this adapter!')
+
+    @classmethod
+    def load_csv_rows(cls, profile, schema, table_name, agate_table):
+        raise dbt.exceptions.NotImplementedException(
+            '`load_csv_rows` is not implemented for this adapter!')
+
     ###
     # FUNCTIONS THAT SHOULD BE ABSTRACT
     ###
     @classmethod
-    def drop(cls, profile, relation, relation_type, model_name=None):
+    def get_result_from_cursor(cls, cursor):
+        data = []
+
+        if cursor.description is not None:
+            column_names = [col[0] for col in cursor.description]
+            raw_results = cursor.fetchall()
+            data = [dict(zip(column_names, row))
+                    for row in raw_results]
+
+        return dbt.clients.agate_helper.table_from_data(data)
+
+    @classmethod
+    def drop(cls, profile, schema, relation, relation_type, model_name=None):
         if relation_type == 'view':
-            return cls.drop_view(profile, relation, model_name)
+            return cls.drop_view(profile, schema, relation, model_name)
         elif relation_type == 'table':
-            return cls.drop_table(profile, relation, model_name)
+            return cls.drop_table(profile, schema, relation, model_name)
         else:
             raise RuntimeError(
                 "Invalid relation_type '{}'"
                 .format(relation_type))
 
     @classmethod
-    def drop_view(cls, profile, view, model_name):
-        schema = cls.get_default_schema(profile)
-
-        sql = ('drop view if exists "{schema}"."{view}" cascade'
-               .format(schema=schema,
-                       view=view))
+    def drop_relation(cls, profile, schema, rel_name, rel_type, model_name):
+        relation = cls.quote_schema_and_table(profile, schema, rel_name)
+        sql = 'drop {} if exists {} cascade'.format(rel_type, relation)
 
         connection, cursor = cls.add_query(profile, sql, model_name)
 
     @classmethod
-    def drop_table(cls, profile, table, model_name):
-        schema = cls.get_default_schema(profile)
+    def drop_view(cls, profile, schema, view, model_name):
+        cls.drop_relation(profile, schema, view, 'view', model_name)
 
-        sql = ('drop table if exists "{schema}"."{table}" cascade'
-               .format(schema=schema,
-                       table=table))
+    @classmethod
+    def drop_table(cls, profile, schema, table, model_name):
+        cls.drop_relation(profile, schema, table, 'table', model_name)
+
+    @classmethod
+    def truncate(cls, profile, schema, table, model_name=None):
+        relation = cls.quote_schema_and_table(profile, schema, table)
+        sql = 'truncate table {}'.format(relation)
 
         connection, cursor = cls.add_query(profile, sql, model_name)
 
     @classmethod
-    def truncate(cls, profile, table, model_name=None):
-        schema = cls.get_default_schema(profile)
-
-        sql = ('truncate table "{schema}"."{table}"'
-               .format(schema=schema,
-                       table=table))
-
-        connection, cursor = cls.add_query(profile, sql, model_name)
-
-    @classmethod
-    def rename(cls, profile, from_name, to_name, model_name=None):
-        schema = cls.get_default_schema(profile)
-
-        sql = ('alter table "{schema}"."{from_name}" rename to "{to_name}"'
-               .format(schema=schema,
-                       from_name=from_name,
-                       to_name=to_name))
+    def rename(cls, profile, schema, from_name, to_name, model_name=None):
+        from_relation = cls.quote_schema_and_table(profile, schema, from_name)
+        to_relation = cls.quote(to_name)
+        sql = 'alter table {} rename to {}'.format(from_relation, to_relation)
 
         connection, cursor = cls.add_query(profile, sql, model_name)
 
@@ -144,35 +172,6 @@ class DefaultAdapter(object):
     def is_cancelable(cls):
         return True
 
-    @classmethod
-    def execute_model(cls, profile, model):
-        parts = re.split(r'-- (DBT_OPERATION .*)', model.get('wrapped_sql'))
-
-        for i, part in enumerate(parts):
-            matches = re.match(r'^DBT_OPERATION ({.*})$', part)
-            if matches is not None:
-                instruction_string = matches.groups()[0]
-                instruction = yaml.safe_load(instruction_string)
-                function = instruction['function']
-                kwargs = instruction['args']
-
-                def call_expand_target_column_types(kwargs):
-                    kwargs.update({'profile': profile,
-                                   'model_name': model.get('name')})
-                    return cls.expand_target_column_types(**kwargs)
-
-                func_map = {
-                    'expand_column_types_if_needed':
-                    call_expand_target_column_types
-                }
-
-                func_map[function](kwargs)
-            else:
-                connection, cursor = cls.add_query(
-                    profile, part, model.get('name'))
-
-        return cls.get_status(cursor)
-
     @classmethod
     def get_missing_columns(cls, profile,
                             from_schema, from_table,
@@ -193,8 +192,7 @@ class DefaultAdapter(object):
                 if col_name in missing_columns]
 
     @classmethod
-    def get_columns_in_table(cls, profile, schema_name, table_name,
-                             model_name=None):
+    def _get_columns_in_table_sql(cls, schema_name, table_name):
         sql = """
         select column_name, data_type, character_maximum_length
         from information_schema.columns
@@ -205,6 +203,13 @@ class DefaultAdapter(object):
             sql += (" AND table_schema = '{schema_name}'"
                     .format(schema_name=schema_name))
 
+        return sql
+
+    @classmethod
+    def get_columns_in_table(cls, profile, schema_name, table_name,
+                             model_name=None):
+
+        sql = cls._get_columns_in_table_sql(schema_name, table_name)
         connection, cursor = cls.add_query(
             profile, sql, model_name)
 
@@ -218,18 +223,23 @@ class DefaultAdapter(object):
 
         return columns
 
+    @classmethod
+    def _table_columns_to_dict(cls, columns):
+        return {col.name: col for col in columns}
+
     @classmethod
     def expand_target_column_types(cls, profile,
                                    temp_table,
                                    to_schema, to_table,
                                    model_name=None):
 
-        reference_columns = {col.name: col for col in
-                             cls.get_columns_in_table(
-                                 profile, None, temp_table, model_name)}
-        target_columns = {col.name: col for col in
-                          cls.get_columns_in_table(
-                              profile, to_schema, to_table, model_name)}
+        reference_columns = cls._table_columns_to_dict(
+            cls.get_columns_in_table(
+                profile, None, temp_table, model_name))
+
+        target_columns = cls._table_columns_to_dict(
+            cls.get_columns_in_table(
+                profile, to_schema, to_table, model_name))
 
         for column_name, reference_column in reference_columns.items():
             target_column = target_columns.get(column_name)
@@ -259,27 +269,6 @@ class DefaultAdapter(object):
         return ('drop schema if exists "{schema} cascade"'
                 .format(schema=schema))
 
-    @classmethod
-    def get_create_table_sql(cls, schema, table, columns, sort, dist):
-        fields = ['"{field}" {data_type}'.format(
-            field=column.name, data_type=column.data_type
-        ) for column in columns]
-        fields_csv = ",\n  ".join(fields)
-        dist = cls.dist_qualifier(dist)
-        sort = cls.sort_qualifier('compound', sort)
-        sql = """
-        create table if not exists "{schema}"."{table}" (
-        {fields}
-        )
-        {dist} {sort}
-        """.format(
-            schema=schema,
-            table=table,
-            fields=fields_csv,
-            sort=sort,
-            dist=dist).strip()
-        return sql
-
     ###
     # ODBC FUNCTIONS -- these should not need to change for every adapter,
     #                   although some adapters may override them
@@ -438,6 +427,10 @@ class DefaultAdapter(object):
     def add_begin_query(cls, profile, name):
         return cls.add_query(profile, 'BEGIN', name, auto_begin=False)
 
+    @classmethod
+    def add_commit_query(cls, profile, name):
+        return cls.add_query(profile, 'COMMIT', name, auto_begin=False)
+
     @classmethod
     def begin(cls, profile, name='master'):
         global connections_in_use
@@ -470,10 +463,10 @@ class DefaultAdapter(object):
 
         connection = cls.get_connection(profile, name, False)
 
-        return cls.commit(connection)
+        return cls.commit(profile, connection)
 
     @classmethod
-    def commit(cls, connection):
+    def commit(cls, profile, connection):
         global connections_in_use
 
         if dbt.flags.STRICT_MODE:
@@ -487,7 +480,7 @@ class DefaultAdapter(object):
                 'it does not have one open!'.format(connection.get('name')))
 
         logger.debug('On {}: COMMIT'.format(connection.get('name')))
-        connection.get('handle').commit()
+        cls.add_commit_query(profile, connection.get('name'))
 
         connection['transaction_open'] = False
         connections_in_use[connection.get('name')] = connection
@@ -532,7 +525,8 @@ class DefaultAdapter(object):
         return connection
 
     @classmethod
-    def add_query(cls, profile, sql, model_name=None, auto_begin=True):
+    def add_query(cls, profile, sql, model_name=None, auto_begin=True,
+                  bindings=None, abridge_sql_log=False):
         connection = cls.get_connection(profile, model_name)
         connection_name = connection.get('name')
 
@@ -543,17 +537,26 @@ class DefaultAdapter(object):
                      .format(cls.type(), connection_name))
 
         with cls.exception_handler(profile, sql, model_name, connection_name):
-            logger.debug('On %s: %s', connection_name, sql)
+            if abridge_sql_log:
+                logger.debug('On %s: %s....', connection_name, sql[0:512])
+            else:
+                logger.debug('On %s: %s', connection_name, sql)
             pre = time.time()
 
             cursor = connection.get('handle').cursor()
-            cursor.execute(sql)
+            cursor.execute(sql, bindings)
 
             logger.debug("SQL status: %s in %0.2f seconds",
                          cls.get_status(cursor), (time.time() - pre))
 
             return connection, cursor
 
+    @classmethod
+    def clear_transaction(cls, profile, conn_name='master'):
+        conn = cls.begin(profile, conn_name)
+        cls.commit(profile, conn)
+        return conn_name
+
     @classmethod
     def execute_one(cls, profile, sql, model_name=None, auto_begin=False):
         cls.get_connection(profile, model_name)
@@ -565,7 +568,19 @@ class DefaultAdapter(object):
                           auto_begin=False):
         _, cursor = cls.execute_one(profile, sql, model_name, auto_begin)
 
-        return cursor.fetchall()
+        status = cls.get_status(cursor)
+        table = cls.get_result_from_cursor(cursor)
+        return status, table
+
+    @classmethod
+    def execute(cls, profile, sql, model_name=None, auto_begin=False,
+                fetch=False):
+        if fetch:
+            return cls.execute_and_fetch(profile, sql, model_name, auto_begin)
+        else:
+            _, cursor = cls.execute_one(profile, sql, model_name, auto_begin)
+            status = cls.get_status(cursor)
+            return status, dbt.clients.agate_helper.empty_table()
 
     @classmethod
     def execute_all(cls, profile, sqls, model_name=None):
@@ -595,13 +610,6 @@ class DefaultAdapter(object):
         sql = cls.get_drop_schema_sql(schema)
         return cls.add_query(profile, sql, model_name)
 
-    @classmethod
-    def create_table(cls, profile, schema, table, columns, sort, dist,
-                     model_name=None):
-        logger.debug('Creating table "%s"."%s".', schema, table)
-        sql = cls.get_create_table_sql(schema, table, columns, sort, dist)
-        return cls.add_query(profile, sql, model_name)
-
     @classmethod
     def table_exists(cls, profile, schema, table, model_name=None):
         tables = cls.query_for_existing(profile, schema, model_name)
@@ -616,5 +624,71 @@ class DefaultAdapter(object):
         return cls.table_exists(profile, schema, table, model_name)
 
     @classmethod
-    def quote_schema_and_table(cls, profile, schema, table):
-        return '"{}"."{}"'.format(schema, table)
+    def quote(cls, identifier):
+        return '"{}"'.format(identifier.replace('"', '""'))
+
+    @classmethod
+    def quote_schema_and_table(cls, profile, schema, table, model_name=None):
+        return '{}.{}'.format(cls.quote(schema),
+                              cls.quote(table))
+
+    @classmethod
+    def handle_csv_table(cls, profile, schema, table_name, agate_table,
+                         full_refresh=False):
+        existing = cls.query_for_existing(profile, schema)
+        existing_type = existing.get(table_name)
+        if existing_type and existing_type != "table":
+            raise dbt.exceptions.RuntimeException(
+                "Cannot seed to '{}', it is a view".format(table_name))
+        if existing_type:
+            cls.reset_csv_table(profile, schema, table_name, agate_table,
+                                full_refresh=full_refresh)
+        else:
+            cls.create_csv_table(profile, schema, table_name, agate_table)
+        cls.load_csv_rows(profile, schema, table_name, agate_table)
+        cls.commit_if_has_connection(profile, None)
+
+    @classmethod
+    def convert_text_type(cls, agate_table, col_idx):
+        raise dbt.exceptions.NotImplementedException(
+            '`convert_text_type` is not implemented for this adapter!')
+
+    @classmethod
+    def convert_number_type(cls, agate_table, col_idx):
+        raise dbt.exceptions.NotImplementedException(
+            '`convert_number_type` is not implemented for this adapter!')
+
+    @classmethod
+    def convert_boolean_type(cls, agate_table, col_idx):
+        raise dbt.exceptions.NotImplementedException(
+            '`convert_boolean_type` is not implemented for this adapter!')
+
+    @classmethod
+    def convert_datetime_type(cls, agate_table, col_idx):
+        raise dbt.exceptions.NotImplementedException(
+            '`convert_datetime_type` is not implemented for this adapter!')
+
+    @classmethod
+    def convert_date_type(cls, agate_table, col_idx):
+        raise dbt.exceptions.NotImplementedException(
+            '`convert_date_type` is not implemented for this adapter!')
+
+    @classmethod
+    def convert_time_type(cls, agate_table, col_idx):
+        raise dbt.exceptions.NotImplementedException(
+            '`convert_time_type` is not implemented for this adapter!')
+
+    @classmethod
+    def convert_agate_type(cls, agate_table, col_idx):
+        agate_type = agate_table.column_types[col_idx]
+        conversions = [
+            (agate.Text, cls.convert_text_type),
+            (agate.Number, cls.convert_number_type),
+            (agate.Boolean, cls.convert_boolean_type),
+            (agate.DateTime, cls.convert_datetime_type),
+            (agate.Date, cls.convert_date_type),
+            (agate.TimeDelta, cls.convert_time_type),
+        ]
+        for agate_cls, func in conversions:
+            if isinstance(agate_type, agate_cls):
+                return func(agate_table, col_idx)
