@@ -7,6 +7,7 @@ import snowflake.connector.errors
 
 from contextlib import contextmanager
 
+import dbt.compat
 import dbt.exceptions
 import dbt.flags as flags
 
@@ -26,26 +27,28 @@ class SnowflakeAdapter(PostgresAdapter):
         try:
             yield
         except snowflake.connector.errors.ProgrammingError as e:
-            logger.debug('Snowflake error: {}'.format(str(e)))
+            msg = dbt.compat.to_string(e)
 
-            if 'Empty SQL statement' in e.msg:
+            logger.debug('Snowflake error: {}'.format(msg))
+
+            if 'Empty SQL statement' in msg:
                 logger.debug("got empty sql statement, moving on")
-            elif 'This session does not have a current database' in e.msg:
-                cls.rollback(connection)
+            elif 'This session does not have a current database' in msg:
+                cls.release_connection(profile, connection_name)
                 raise dbt.exceptions.FailedToConnectException(
                     ('{}\n\nThis error sometimes occurs when invalid '
                      'credentials are provided, or when your default role '
                      'does not have access to use the specified database. '
                      'Please double check your profile and try again.')
-                    .format(str(e)))
+                    .format(msg))
             else:
-                cls.rollback(connection)
-                raise dbt.exceptions.ProgrammingException(str(e))
+                cls.release_connection(profile, connection_name)
+                raise dbt.exceptions.DatabaseException(msg)
         except Exception as e:
             logger.debug("Error running SQL: %s", sql)
             logger.debug("Rolling back transaction.")
-            cls.rollback(connection)
-            raise e
+            cls.release_connection(profile, connection_name)
+            raise dbt.exceptions.RuntimeException(e.msg)
 
     @classmethod
     def type(cls):
@@ -100,12 +103,17 @@ class SnowflakeAdapter(PostgresAdapter):
         return result
 
     @classmethod
-    def query_for_existing(cls, profile, schema, model_name=None):
+    def query_for_existing(cls, profile, schemas, model_name=None):
+        if not isinstance(schemas, (list, tuple)):
+            schemas = [schemas]
+
+        schema_list = ",".join(["'{}'".format(schema) for schema in schemas])
+
         sql = """
         select TABLE_NAME as name, TABLE_TYPE as type
         from INFORMATION_SCHEMA.TABLES
-        where TABLE_SCHEMA = '{schema}'
-        """.format(schema=schema).strip()  # noqa
+        where TABLE_SCHEMA in ({schema_list})
+        """.format(schema_list=schema_list).strip()  # noqa
 
         _, cursor = cls.add_query(profile, sql, model_name, auto_begin=False)
         results = cursor.fetchall()
@@ -121,9 +129,7 @@ class SnowflakeAdapter(PostgresAdapter):
         return dict(existing)
 
     @classmethod
-    def rename(cls, profile, from_name, to_name, model_name=None):
-        schema = cls.get_default_schema(profile)
-
+    def rename(cls, profile, schema, from_name, to_name, model_name=None):
         sql = (('alter table "{schema}"."{from_name}" '
                 'rename to "{schema}"."{to_name}"')
                .format(schema=schema,
@@ -132,16 +138,6 @@ class SnowflakeAdapter(PostgresAdapter):
 
         connection, cursor = cls.add_query(profile, sql, model_name)
 
-    @classmethod
-    def execute_model(cls, profile, model):
-        connection = cls.get_connection(profile, model.get('name'))
-
-        if flags.STRICT_MODE:
-            validate_connection(connection)
-
-        return super(PostgresAdapter, cls).execute_model(
-            profile, model)
-
     @classmethod
     def add_begin_query(cls, profile, name):
         return cls.add_query(profile, 'BEGIN', name, auto_begin=False,
@@ -151,7 +147,22 @@ class SnowflakeAdapter(PostgresAdapter):
     def create_schema(cls, profile, schema, model_name=None):
         logger.debug('Creating schema "%s".', schema)
         sql = cls.get_create_schema_sql(schema)
-        return cls.add_query(profile, sql, model_name, select_schema=False)
+        res = cls.add_query(profile, sql, model_name, select_schema=False)
+
+        cls.commit_if_has_connection(profile, model_name)
+
+        return res
+
+    @classmethod
+    def get_existing_schemas(cls, profile, model_name=None):
+        sql = "select distinct SCHEMA_NAME from INFORMATION_SCHEMA.SCHEMATA"
+
+        connection, cursor = cls.add_query(profile, sql, model_name,
+                                           select_schema=False,
+                                           auto_begin=False)
+        results = cursor.fetchall()
+
+        return [row[0] for row in results]
 
     @classmethod
     def check_schema_exists(cls, profile, schema, model_name=None):
@@ -168,19 +179,18 @@ class SnowflakeAdapter(PostgresAdapter):
 
         return results[0] > 0
 
+    # TODO remove select_schema
     @classmethod
     def add_query(cls, profile, sql, model_name=None, auto_begin=True,
-                  select_schema=True):
+                  select_schema=True, bindings=None, abridge_sql_log=False):
         # snowflake only allows one query per api call.
         queries = sql.strip().split(";")
         cursor = None
 
-        if select_schema:
-            super(PostgresAdapter, cls).add_query(
-                profile,
-                'use schema "{}"'.format(cls.get_default_schema(profile)),
-                model_name,
-                auto_begin)
+        if bindings:
+            # The snowflake connector is more strict than, eg., psycopg2 -
+            # which allows any iterable thing to be passed as a binding.
+            bindings = tuple(bindings)
 
         for individual_query in queries:
             # hack -- after the last ';', remove comments and don't run
@@ -194,7 +204,8 @@ class SnowflakeAdapter(PostgresAdapter):
                 continue
 
             connection, cursor = super(PostgresAdapter, cls).add_query(
-                profile, individual_query, model_name, auto_begin)
+                profile, individual_query, model_name, auto_begin,
+                bindings=bindings, abridge_sql_log=abridge_sql_log)
 
         return connection, cursor
 
