@@ -6,15 +6,14 @@ import sqlparse
 import dbt.project
 import dbt.utils
 import dbt.include
-import dbt.wrapper
 import dbt.tracking
 
-from dbt.utils import This, Var, get_materialization, is_type
-from dbt.node_types import NodeType
+from dbt.utils import get_materialization, NodeType, is_type
 
 from dbt.linker import Linker
 
 import dbt.compat
+import dbt.context.runtime
 import dbt.contracts.graph.compiled
 import dbt.contracts.project
 import dbt.exceptions
@@ -22,13 +21,8 @@ import dbt.flags
 import dbt.loader
 import dbt.parser
 
-from dbt.adapters.factory import get_adapter
 from dbt.logger import GLOBAL_LOGGER as logger
 
-CompilableEntities = [
-    "models", "data tests", "schema tests", "archives", "analyses"
-]
-
 graph_file_name = 'graph.gpickle'
 
 
@@ -40,17 +34,10 @@ def print_compile_stats(stats):
         NodeType.Analysis: 'analyses',
         NodeType.Macro: 'macros',
         NodeType.Operation: 'operations',
+        NodeType.Seed: 'seed files',
     }
 
-    results = {
-        NodeType.Model: 0,
-        NodeType.Test: 0,
-        NodeType.Archive: 0,
-        NodeType.Analysis: 0,
-        NodeType.Macro: 0,
-        NodeType.Operation: 0,
-    }
-
+    results = {k: 0 for k in names.keys()}
     results.update(stats)
 
     stat_line = ", ".join(
@@ -153,7 +140,6 @@ def inject_ctes_into_sql(sql, ctes):
 class Compiler(object):
     def __init__(self, project):
         self.project = project
-        self.parsed_models = None
 
     def initialize(self):
         dbt.clients.system.make_directory(self.project['target-path'])
@@ -167,101 +153,6 @@ class Compiler(object):
 
         return target_path
 
-    def __model_config(self, model):
-        def do_config(*args, **kwargs):
-            return ''
-
-        return do_config
-
-    def __ref(self, ctx, model, flat_graph, current_project):
-        schema = ctx.get('env', {}).get('schema')
-
-        def do_ref(*args):
-            target_model_name = None
-            target_model_package = None
-
-            if len(args) == 1:
-                target_model_name = args[0]
-            elif len(args) == 2:
-                target_model_package, target_model_name = args
-            else:
-                dbt.exceptions.ref_invalid_args(model, args)
-
-            target_model = dbt.parser.resolve_ref(
-                flat_graph,
-                target_model_name,
-                target_model_package,
-                current_project,
-                model.get('package_name'))
-
-            if target_model is None:
-                dbt.exceptions.ref_target_not_found(
-                    model,
-                    target_model_name,
-                    target_model_package)
-
-            target_model_id = target_model.get('unique_id')
-
-            if target_model_id not in model.get('depends_on', {}).get('nodes'):
-                dbt.exceptions.ref_bad_context(model,
-                                               target_model_name,
-                                               target_model_package)
-
-            if get_materialization(target_model) == 'ephemeral':
-                model['extra_ctes'][target_model_id] = None
-                return '__dbt__CTE__{}'.format(target_model.get('name'))
-            else:
-                profile = self.project.run_environment()
-                adapter = get_adapter(profile)
-                table = target_model.get('name')
-                return adapter.quote_schema_and_table(profile, schema, table)
-
-        return do_ref
-
-    def get_compiler_context(self, model, flat_graph):
-        context = self.project.context()
-        profile = self.project.run_environment()
-        adapter = get_adapter(profile)
-
-        wrapper = dbt.wrapper.DatabaseWrapper(model, adapter, profile)
-
-        # built-ins
-        context['ref'] = self.__ref(context, model, flat_graph,
-                                    self.project.cfg.get('name'))
-        context['config'] = self.__model_config(model)
-        context['this'] = This(
-            context['env']['schema'],
-            dbt.utils.model_immediate_name(model, dbt.flags.NON_DESTRUCTIVE),
-            model.get('name')
-        )
-        context['var'] = Var(model, context=context)
-        context['target'] = self.project.get_target()
-        context['adapter'] = wrapper
-        context['flags'] = dbt.flags
-
-        context.update(wrapper.get_context_functions())
-
-        context['run_started_at'] = dbt.tracking.active_user.run_started_at
-        context['invocation_id'] = dbt.tracking.active_user.invocation_id
-        context['sql_now'] = adapter.date_function()
-
-        for unique_id, macro in flat_graph.get('macros').items():
-            package_name = macro.get('package_name')
-
-            macro_map = {macro.get('name'): macro.get('parsed_macro')}
-
-            if context.get(package_name) is None:
-                context[package_name] = {}
-
-            context.get(package_name, {}) \
-                   .update(macro_map)
-
-            if(package_name == model.get('package_name') or
-               package_name == dbt.include.GLOBAL_PROJECT_NAME):
-                context.update(macro_map)
-
-        return context
-
     def compile_node(self, node, flat_graph):
         logger.debug("Compiling {}".format(node.get('unique_id')))
 
@@ -274,7 +165,8 @@ class Compiler(object):
             'injected_sql': None,
         })
 
-        context = self.get_compiler_context(compiled_node, flat_graph)
+        context = dbt.context.runtime.generate(
+            compiled_node, self.project.cfg, flat_graph)
 
         compiled_node['compiled_sql'] = dbt.clients.jinja.get_rendered(
             node.get('raw_sql'),
@@ -311,23 +203,7 @@ class Compiler(object):
             pass
 
         else:
-            wrapped_stmt = dbt.wrapper.wrap(
-                injected_node,
-                self.project,
-                context,
-                flat_graph)
-
-            injected_node['wrapped_sql'] = wrapped_stmt
-
-        if 'wrapped_sql' in injected_node:
-            build_path = os.path.join('build',
-                                      injected_node.get('package_name'),
-                                      injected_node.get('path'))
-
-            written_path = self.__write(build_path,
-                                        injected_node['wrapped_sql'])
-
-            injected_node['build_path'] = written_path
+            injected_node['wrapped_sql'] = None
 
         return injected_node
 
