@@ -1,28 +1,17 @@
 
 {% macro test_unique(model, arg) %}
 
-with validation as (
+select count(*)
+from (
 
     select
-        {{ arg }} as unique_field
+        {{ arg }}
 
     from {{ model }}
     where {{ arg }} is not null
-
-),
-
-validation_errors as (
-
-    select
-        unique_field
-
-    from validation
-    group by unique_field
+    group by {{ arg }}
     having count(*) > 1
 
-)
-
-select count(*)
-from validation_errors
+) validation_errors
 
 {% endmacro %}
