@@ -3,6 +3,7 @@ from dbt.logger import initialize_logger, GLOBAL_LOGGER as logger
 import argparse
 import os.path
 import sys
+import traceback
 
 import dbt.version
 import dbt.flags as flags
@@ -21,13 +22,14 @@ import dbt.tracking
 import dbt.config as config
 import dbt.ui.printer
 import dbt.compat
+import dbt.deprecations
 
 from dbt.utils import ExitCodes
 
 PROFILES_HELP_MESSAGE = """
 For more information on configuring profiles, please consult the dbt docs:
 
-https://dbt.readme.io/docs/configure-your-profile
+https://docs.getdbt.com/docs/configure-your-profile
 """
 
 
@@ -53,6 +55,8 @@ def main(args=None):
     except BaseException as e:
         logger.info("Encountered an error:")
         logger.info(str(e))
+
+        logger.debug(traceback.format_exc())
         exit_code = ExitCodes.UnhandledError
 
     sys.exit(exit_code)
@@ -211,6 +215,8 @@ def invoke_dbt(parsed):
         targets = proj.cfg.get('outputs', {}).keys()
         if parsed.target in targets:
             proj.cfg['target'] = parsed.target
+            # make sure we update the target if this is overriden on the cli
+            proj.compile_and_update_target()
         else:
             logger.info("Encountered an error while reading the project:")
             logger.info("  ERROR Specified target {} is not a valid option "
@@ -227,7 +233,15 @@ def invoke_dbt(parsed):
             return None
 
     flags.NON_DESTRUCTIVE = getattr(proj.args, 'non_destructive', False)
-    flags.FULL_REFRESH = getattr(proj.args, 'full_refresh', False)
+
+    arg_drop_existing = getattr(proj.args, 'drop_existing', False)
+    arg_full_refresh = getattr(proj.args, 'full_refresh', False)
+
+    if arg_drop_existing:
+        dbt.deprecations.warn('drop-existing')
+        flags.FULL_REFRESH = True
+    elif arg_full_refresh:
+        flags.FULL_REFRESH = True
 
     logger.debug("running dbt with arguments %s", parsed)
 
@@ -290,6 +304,16 @@ def parse_args(args):
         help='Which target to load for the given profile'
     )
 
+    base_subparser.add_argument(
+        '--vars',
+        type=str,
+        default='{}',
+        help="""
+            Supply variables to the project. This argument overrides
+            variables defined in your dbt_project.yml file. This argument
+            should be a YAML string, eg. '{my_variable: my_value}'"""
+    )
+
     sub = subs.add_parser('init', parents=[base_subparser])
     sub.add_argument('project_name', type=str, help='Name of the new project')
     sub.set_defaults(cls=init_task.InitTask, which='init')
@@ -370,13 +394,23 @@ def parse_args(args):
             fully-recalculate the incremental table from the model definition.
             """)
 
-    sub = subs.add_parser('seed', parents=[base_subparser])
-    sub.add_argument(
+    seed_sub = subs.add_parser('seed', parents=[base_subparser])
+    seed_sub.add_argument(
         '--drop-existing',
         action='store_true',
-        help="Drop existing seed tables and recreate them"
+        help='(DEPRECATED) Use --full-refresh instead.'
     )
-    sub.set_defaults(cls=seed_task.SeedTask, which='seed')
+    seed_sub.add_argument(
+        '--full-refresh',
+        action='store_true',
+        help='Drop existing seed tables and recreate them'
+    )
+    seed_sub.add_argument(
+        '--show',
+        action='store_true',
+        help='Show a sample of the loaded data in the terminal'
+    )
+    seed_sub.set_defaults(cls=seed_task.SeedTask, which='seed')
 
     sub = subs.add_parser('test', parents=[base_subparser])
     sub.add_argument(
