@@ -1,75 +1,45 @@
 import copy
 import os
 import re
+import hashlib
+import collections
 
+import dbt.exceptions
 import dbt.flags
 import dbt.model
 import dbt.utils
+import dbt.hooks
 
 import jinja2.runtime
 import dbt.clients.jinja
 import dbt.clients.yaml_helper
+import dbt.clients.agate_helper
+
+import dbt.context.parser
 
 import dbt.contracts.graph.parsed
 import dbt.contracts.graph.unparsed
 import dbt.contracts.project
 
-from dbt.utils import Var
 from dbt.node_types import NodeType, RunHookType
 from dbt.compat import basestring, to_string
 from dbt.logger import GLOBAL_LOGGER as logger
+from dbt.utils import get_pseudo_test_path, coalesce
 
 
 def get_path(resource_type, package_name, resource_name):
     return "{}.{}.{}".format(resource_type, package_name, resource_name)
 
 
-def get_model_path(package_name, resource_name):
-    return get_path(NodeType.Model, package_name, resource_name)
-
-
 def get_test_path(package_name, resource_name):
     return get_path(NodeType.Test, package_name, resource_name)
 
 
-def get_macro_path(package_name, resource_name):
-    return get_path('macros', package_name, resource_name)
-
-
-def __config(model, cfg):
-
-    def config(*args, **kwargs):
-        if len(args) == 1 and len(kwargs) == 0:
-            opts = args[0]
-        elif len(args) == 0 and len(kwargs) > 0:
-            opts = kwargs
-        else:
-            dbt.utils.compiler_error(
-                model.get('name'),
-                "Invalid model config given inline in {}".format(model))
-
-        cfg.update_in_model_config(opts)
-
-    return config
-
-
-def __ref(model):
-
-    def ref(*args):
-        if len(args) == 1 or len(args) == 2:
-            model['refs'].append(args)
-
-        else:
-            dbt.exceptions.ref_invalid_args(model, args)
-
-    return ref
-
-
 def resolve_ref(flat_graph, target_model_name, target_model_package,
                 current_project, node_package):
 
     if target_model_package is not None:
-        return dbt.utils.find_model_by_name(
+        return dbt.utils.find_refable_by_name(
             flat_graph,
             target_model_name,
             target_model_package)
@@ -77,7 +47,7 @@ def resolve_ref(flat_graph, target_model_name, target_model_package,
     target_model = None
 
     # first pass: look for models in the current_project
-    target_model = dbt.utils.find_model_by_name(
+    target_model = dbt.utils.find_refable_by_name(
         flat_graph,
         target_model_name,
         current_project)
@@ -86,7 +56,7 @@ def resolve_ref(flat_graph, target_model_name, target_model_package,
         return target_model
 
     # second pass: look for models in the node's package
-    target_model = dbt.utils.find_model_by_name(
+    target_model = dbt.utils.find_refable_by_name(
         flat_graph,
         target_model_name,
         node_package)
@@ -97,7 +67,7 @@ def resolve_ref(flat_graph, target_model_name, target_model_package,
     # final pass: look for models in any package
     # todo: exclude the packages we have already searched. overriding
     # a package model in another package doesn't necessarily work atm
-    return dbt.utils.find_model_by_name(
+    return dbt.utils.find_refable_by_name(
         flat_graph,
         target_model_name,
         None)
@@ -123,17 +93,13 @@ def process_refs(flat_graph, current_project):
                 node.get('package_name'))
 
             if target_model is None:
-                dbt.exceptions.ref_target_not_found(
-                    node,
-                    target_model_name,
-                    target_model_package)
-
-            if (dbt.utils.is_enabled(node) and not
-                    dbt.utils.is_enabled(target_model)):
-                if dbt.utils.is_type(node, NodeType.Model):
-                    dbt.exceptions.ref_disabled_dependency(node, target_model)
-                else:
-                    node.get('config', {})['enabled'] = False
+                # This may raise. Even if it doesn't, we don't want to add
+                # this node to the graph b/c there is no destination node
+                node.get('config', {})['enabled'] = False
+                dbt.utils.invalid_ref_fail_unless_test(node,
+                                                       target_model_name,
+                                                       target_model_package)
+                continue
 
             target_model_id = target_model.get('unique_id')
 
@@ -171,8 +137,8 @@ def parse_macro_file(macro_file_path,
     context = {}
 
     base_node = {
-        'name': 'macro',
         'path': macro_file_path,
+        'original_file_path': macro_file_path,
         'resource_type': NodeType.Macro,
         'package_name': package_name,
         'depends_on': {
@@ -180,40 +146,48 @@ def parse_macro_file(macro_file_path,
         }
     }
 
-    template = dbt.clients.jinja.get_template(
-        macro_file_contents, context, node=base_node, validate_macro=True)
+    try:
+        template = dbt.clients.jinja.get_template(
+            macro_file_contents, context, node=base_node)
+    except dbt.exceptions.CompilationException as e:
+        e.node = base_node
+        raise e
 
     for key, item in template.module.__dict__.items():
         if type(item) == jinja2.runtime.Macro:
+            name = key.replace(dbt.utils.MACRO_PREFIX, '')
+
             unique_id = get_path(NodeType.Macro,
                                  package_name,
-                                 key)
+                                 name)
 
             new_node = base_node.copy()
             new_node.update({
-                'name': key,
+                'name': name,
                 'unique_id': unique_id,
                 'tags': tags,
                 'root_path': root_path,
                 'path': macro_file_path,
+                'original_file_path': macro_file_path,
                 'raw_sql': macro_file_contents,
-                'parsed_macro': item
             })
+
+            new_node['generator'] = dbt.clients.jinja.macro_generator(
+                template, new_node)
+
             to_return[unique_id] = new_node
 
     return to_return
 
 
 def parse_node(node, node_path, root_project_config, package_project_config,
-               all_projects, tags=None, fqn_extra=None):
+               all_projects, tags=None, fqn_extra=None, fqn=None, macros=None):
     logger.debug("Parsing {}".format(node_path))
     node = copy.deepcopy(node)
 
-    if tags is None:
-        tags = set()
-
-    if fqn_extra is None:
-        fqn_extra = []
+    tags = coalesce(tags, set())
+    fqn_extra = coalesce(fqn_extra, [])
+    macros = coalesce(macros, {})
 
     node.update({
         'refs': [],
@@ -223,15 +197,20 @@ def parse_node(node, node_path, root_project_config, package_project_config,
         }
     })
 
-    fqn = get_fqn(node.get('path'), package_project_config, fqn_extra)
+    if fqn is None:
+        fqn = get_fqn(node.get('path'), package_project_config, fqn_extra)
 
     config = dbt.model.SourceConfig(
-        root_project_config, package_project_config, fqn)
+        root_project_config,
+        package_project_config,
+        fqn,
+        node['resource_type'])
 
     node['unique_id'] = node_path
-    node['empty'] = (len(node.get('raw_sql').strip()) == 0)
+    node['empty'] = ('raw_sql' in node and len(node['raw_sql'].strip()) == 0)
     node['fqn'] = fqn
     node['tags'] = tags
+    node['config_reference'] = config
 
     # Set this temporarily. Not the full config yet (as config() hasn't been
     # called from jinja yet). But the Var() call below needs info about project
@@ -240,30 +219,49 @@ def parse_node(node, node_path, root_project_config, package_project_config,
     config_dict.update(config.config)
     node['config'] = config_dict
 
-    context = {}
-    context['ref'] = __ref(node)
-    context['config'] = __config(node, config)
-    context['target'] = property(lambda x: '', lambda x: x)
-    context['this'] = ''
+    # Set this temporarily so get_rendered() below has access to a schema
+    profile = dbt.utils.get_profile_from_project(root_project_config)
+    default_schema = profile.get('schema', 'public')
+    node['schema'] = default_schema
 
-    context['var'] = Var(node, context)
+    context = dbt.context.parser.generate(node, root_project_config,
+                                          {"macros": macros})
 
     dbt.clients.jinja.get_rendered(
         node.get('raw_sql'), context, node,
         capture_macros=True)
 
+    # Clean up any open connections opened by adapter functions that hit the db
+    db_wrapper = context['adapter']
+    adapter = db_wrapper.adapter
+    profile = db_wrapper.profile
+    adapter.release_connection(profile, node.get('name'))
+
+    # Special macro defined in the global project
+    schema_override = config.config.get('schema')
+    get_schema = context.get('generate_schema_name', lambda x: default_schema)
+    node['schema'] = get_schema(schema_override)
+
     # Overwrite node config
     config_dict = node.get('config', {})
     config_dict.update(config.config)
     node['config'] = config_dict
 
+    for hook_type in dbt.hooks.ModelHookType.Both:
+        node['config'][hook_type] = dbt.hooks.get_hooks(node, hook_type)
+
+    del node['config_reference']
+
     return node
 
 
-def parse_sql_nodes(nodes, root_project, projects, tags=None):
+def parse_sql_nodes(nodes, root_project, projects, tags=None, macros=None):
     if tags is None:
         tags = set()
 
+    if macros is None:
+        macros = {}
+
     to_return = {}
 
     dbt.contracts.graph.unparsed.validate_nodes(nodes)
@@ -275,13 +273,27 @@ def parse_sql_nodes(nodes, root_project, projects, tags=None):
                              package_name,
                              node.get('name'))
 
-        # TODO if this is set, raise a compiler error
-        to_return[node_path] = parse_node(node,
-                                          node_path,
-                                          root_project,
-                                          projects.get(package_name),
-                                          projects,
-                                          tags=tags)
+        node_parsed = parse_node(node,
+                                 node_path,
+                                 root_project,
+                                 projects.get(package_name),
+                                 projects,
+                                 tags=tags,
+                                 macros=macros)
+
+        # Ignore disabled nodes
+        if not node_parsed['config']['enabled']:
+            continue
+
+        # Check for duplicate model names
+        existing_node = to_return.get(node_path)
+        if existing_node is not None:
+            raise dbt.exceptions.CompilationException(
+                'Found models with the same name:\n- %s\n- %s' % (
+                    existing_node.get('original_file_path'),
+                    node.get('original_file_path')))
+
+        to_return[node_path] = node_parsed
 
     dbt.contracts.graph.parsed.validate_nodes(to_return)
 
@@ -289,12 +301,15 @@ def parse_sql_nodes(nodes, root_project, projects, tags=None):
 
 
 def load_and_parse_sql(package_name, root_project, all_projects, root_dir,
-                       relative_dirs, resource_type, tags=None):
+                       relative_dirs, resource_type, tags=None, macros=None):
     extension = "[!.#~]*.sql"
 
     if tags is None:
         tags = set()
 
+    if macros is None:
+        macros = {}
+
     if dbt.flags.STRICT_MODE:
         dbt.contracts.project.validate_list(all_projects)
 
@@ -320,16 +335,21 @@ def load_and_parse_sql(package_name, root_project, all_projects, root_dir,
         else:
             path = file_match.get('relative_path')
 
+        original_file_path = os.path.join(
+            file_match.get('searched_path'),
+            path)
+
         result.append({
             'name': name,
             'root_path': root_dir,
             'resource_type': resource_type,
             'path': path,
+            'original_file_path': original_file_path,
             'package_name': package_name,
             'raw_sql': file_contents
         })
 
-    return parse_sql_nodes(result, root_project, all_projects, tags)
+    return parse_sql_nodes(result, root_project, all_projects, tags, macros)
 
 
 def get_hooks_from_project(project_cfg, hook_type):
@@ -342,18 +362,17 @@ def get_hooks_from_project(project_cfg, hook_type):
 
 
 def get_hooks(all_projects, hook_type):
-    project_hooks = {}
+    project_hooks = collections.defaultdict(list)
 
     for project_name, project in all_projects.items():
         hooks = get_hooks_from_project(project, hook_type)
-
-        if len(hooks) > 0:
-            project_hooks[project_name] = ";\n".join(hooks)
+        project_hooks[project_name].extend(hooks)
 
     return project_hooks
 
 
-def load_and_parse_run_hook_type(root_project, all_projects, hook_type):
+def load_and_parse_run_hook_type(root_project, all_projects, hook_type,
+                                 macros=None):
 
     if dbt.flags.STRICT_MODE:
         dbt.contracts.project.validate_list(all_projects)
@@ -364,27 +383,36 @@ def load_and_parse_run_hook_type(root_project, all_projects, hook_type):
     for project_name, hooks in project_hooks.items():
         project = all_projects[project_name]
 
-        hook_path = dbt.utils.get_pseudo_hook_path(hook_type)
+        for i, hook in enumerate(hooks):
+            hook_name = '{}-{}-{}'.format(project_name, hook_type, i)
+            hook_path = dbt.utils.get_pseudo_hook_path(hook_name)
 
-        result.append({
-            'name': hook_type,
-            'root_path': "{}/dbt_project.yml".format(project_name),
-            'resource_type': NodeType.Operation,
-            'path': hook_path,
-            'package_name': project_name,
-            'raw_sql': hooks
-        })
+            result.append({
+                'name': hook_name,
+                'root_path': "{}/dbt_project.yml".format(project_name),
+                'resource_type': NodeType.Operation,
+                'path': hook_path,
+                'original_file_path': hook_path,
+                'package_name': project_name,
+                'raw_sql': hook,
+                'index': i
+            })
 
     tags = {hook_type}
-    return parse_sql_nodes(result, root_project, all_projects, tags=tags)
+    return parse_sql_nodes(result, root_project, all_projects, tags=tags,
+                           macros=macros)
 
 
-def load_and_parse_run_hooks(root_project, all_projects):
+def load_and_parse_run_hooks(root_project, all_projects, macros=None):
+    if macros is None:
+        macros = {}
+
     hook_nodes = {}
     for hook_type in RunHookType.Both:
         project_hooks = load_and_parse_run_hook_type(root_project,
                                                      all_projects,
-                                                     hook_type)
+                                                     hook_type,
+                                                     macros=macros)
         hook_nodes.update(project_hooks)
 
     return hook_nodes
@@ -423,7 +451,37 @@ def load_and_parse_macros(package_name, root_project, all_projects, root_dir,
     return result
 
 
-def parse_schema_tests(tests, root_project, projects):
+def get_parsed_schema_test(test_node, test_type, model_name, config,
+                           root_project, projects, macros):
+
+    package_name = test_node.get('package_name')
+    test_namespace = None
+    original_test_type = test_type
+    split = test_type.split('.')
+
+    if len(split) > 1:
+        test_type = split[1]
+        package_name = split[0]
+        test_namespace = package_name
+
+    source_package = projects.get(package_name)
+    if source_package is None:
+        desc = '"{}" test on model "{}"'.format(original_test_type, model_name)
+        dbt.exceptions.raise_dep_not_found(test_node, desc, test_namespace)
+
+    return parse_schema_test(
+        test_node,
+        model_name,
+        config,
+        test_namespace,
+        test_type,
+        root_project,
+        source_package,
+        all_projects=projects,
+        macros=macros)
+
+
+def parse_schema_tests(tests, root_project, projects, macros=None):
     to_return = {}
 
     for test in tests:
@@ -439,8 +497,12 @@ def parse_schema_tests(tests, root_project, projects):
         if test_yml is None:
             continue
 
+        no_tests_warning = ("* WARNING: No constraints found for model"
+                            " '{}' in file {}\n")
         for model_name, test_spec in test_yml.items():
             if test_spec is None or test_spec.get('constraints') is None:
+                test_path = test.get('original_file_path', '<unknown>')
+                logger.warning(no_tests_warning.format(model_name, test_path))
                 continue
 
             for test_type, configs in test_spec.get('constraints', {}).items():
@@ -457,11 +519,9 @@ def parse_schema_tests(tests, root_project, projects):
                     continue
 
                 for config in configs:
-                    to_add = parse_schema_test(
-                        test, model_name, config, test_type,
-                        root_project,
-                        projects.get(test.get('package_name')),
-                        all_projects=projects)
+                    to_add = get_parsed_schema_test(
+                                test, test_type, model_name, config,
+                                root_project, projects, macros)
 
                     if to_add is not None:
                         to_return[to_add.get('unique_id')] = to_add
@@ -486,7 +546,17 @@ def get_nice_schema_test_name(test_type, test_name, args):
 
     clean_flat_args = [re.sub('[^0-9a-zA-Z_]+', '_', arg) for arg in flat_args]
     unique = "__".join(clean_flat_args)
-    return '{}_{}_{}'.format(test_type, test_name, unique)
+
+    cutoff = 32
+    if len(unique) <= cutoff:
+        label = unique
+    else:
+        label = hashlib.md5(unique.encode('utf-8')).hexdigest()
+
+    filename = '{}_{}_{}'.format(test_type, test_name, label)
+    name = '{}_{}_{}'.format(test_type, test_name, unique)
+
+    return filename, name
 
 
 def as_kwarg(key, value):
@@ -502,9 +572,9 @@ def as_kwarg(key, value):
     return "{key}={value}".format(key=key, value=formatted_value)
 
 
-def parse_schema_test(test_base, model_name, test_config, test_type,
-                      root_project_config, package_project_config,
-                      all_projects):
+def parse_schema_test(test_base, model_name, test_config, test_namespace,
+                      test_type, root_project_config, package_project_config,
+                      all_projects, macros=None):
 
     if isinstance(test_config, (basestring, int, float, bool)):
         test_args = {'arg': test_config}
@@ -514,37 +584,51 @@ def parse_schema_test(test_base, model_name, test_config, test_type,
     # sort the dict so the keys are rendered deterministically (for tests)
     kwargs = [as_kwarg(key, test_args[key]) for key in sorted(test_args)]
 
+    if test_namespace is None:
+        macro_name = "test_{}".format(test_type)
+    else:
+        macro_name = "{}.test_{}".format(test_namespace, test_type)
+
     raw_sql = "{{{{ {macro}(model=ref('{model}'), {kwargs}) }}}}".format(**{
         'model': model_name,
-        'macro': "test_{}".format(test_type),
+        'macro': macro_name,
         'kwargs': ", ".join(kwargs)
     })
 
-    name = get_nice_schema_test_name(test_type, model_name, test_args)
+    base_path = test_base.get('path')
+    hashed_name, full_name = get_nice_schema_test_name(test_type, model_name,
+                                                       test_args)
+
+    hashed_path = get_pseudo_test_path(hashed_name, base_path, 'schema_test')
+    full_path = get_pseudo_test_path(full_name, base_path, 'schema_test')
+
+    # supply our own fqn which overrides the hashed version from the path
+    fqn_override = get_fqn(full_path, package_project_config)
 
-    pseudo_path = dbt.utils.get_pseudo_test_path(name, test_base.get('path'),
-                                                 'schema_test')
     to_return = {
-        'name': name,
+        'name': full_name,
         'resource_type': test_base.get('resource_type'),
         'package_name': test_base.get('package_name'),
         'root_path': test_base.get('root_path'),
-        'path': pseudo_path,
+        'path': hashed_path,
+        'original_file_path': test_base.get('original_file_path'),
         'raw_sql': raw_sql
     }
 
     return parse_node(to_return,
                       get_test_path(test_base.get('package_name'),
-                                    name),
+                                    full_name),
                       root_project_config,
                       package_project_config,
                       all_projects,
                       tags={'schema'},
-                      fqn_extra=None)
+                      fqn_extra=None,
+                      fqn=fqn_override,
+                      macros=macros)
 
 
 def load_and_parse_yml(package_name, root_project, all_projects, root_dir,
-                       relative_dirs):
+                       relative_dirs, macros=None):
     extension = "[!.#~]*.yml"
 
     if dbt.flags.STRICT_MODE:
@@ -561,6 +645,9 @@ def load_and_parse_yml(package_name, root_project, all_projects, root_dir,
         file_contents = dbt.clients.system.load_file_contents(
             file_match.get('absolute_path'), strip=False)
 
+        original_file_path = os.path.join(file_match.get('searched_path'),
+                                          file_match.get('relative_path'))
+
         parts = dbt.utils.split_path(file_match.get('relative_path', ''))
         name, _ = os.path.splitext(parts[-1])
 
@@ -569,14 +656,15 @@ def load_and_parse_yml(package_name, root_project, all_projects, root_dir,
             'root_path': root_dir,
             'resource_type': NodeType.Test,
             'path': file_match.get('relative_path'),
+            'original_file_path': original_file_path,
             'package_name': package_name,
             'raw_yml': file_contents
         })
 
-    return parse_schema_tests(result, root_project, all_projects)
+    return parse_schema_tests(result, root_project, all_projects, macros)
 
 
-def parse_archives_from_projects(root_project, all_projects):
+def parse_archives_from_projects(root_project, all_projects, macros=None):
     archives = []
     to_return = {}
 
@@ -593,7 +681,8 @@ def parse_archives_from_projects(root_project, all_projects):
             node_path,
             root_project,
             all_projects.get(archive.get('package_name')),
-            all_projects)
+            all_projects,
+            macros=macros)
 
     return to_return
 
@@ -619,9 +708,61 @@ def parse_archives_from_project(project):
                 'root_path': project.get('project-root'),
                 'resource_type': NodeType.Archive,
                 'path': os.path.join('archive', *fake_path),
+                'original_file_path': 'dbt_project.yml',
                 'package_name': project.get('name'),
                 'config': config,
-                'raw_sql': '-- noop'
+                'raw_sql': '{{config(materialized="archive")}} -- noop'
             })
 
     return archives
+
+
+def parse_seed_file(file_match, root_dir, package_name):
+    abspath = file_match['absolute_path']
+    logger.debug("Parsing {}".format(abspath))
+    to_return = {}
+    table_name = os.path.basename(abspath)[:-4]
+    node = {
+        'unique_id': get_path(NodeType.Seed, package_name, table_name),
+        'path': file_match['relative_path'],
+        'name': table_name,
+        'root_path': root_dir,
+        'resource_type': NodeType.Seed,
+        # Give this raw_sql so it conforms to the node spec,
+        # use dummy text so it doesn't look like an empty node
+        'raw_sql': '-- csv --',
+        'package_name': package_name,
+        'depends_on': {'nodes': []},
+        'original_file_path': os.path.join(file_match.get('searched_path'),
+                                           file_match.get('relative_path')),
+    }
+    try:
+        table = dbt.clients.agate_helper.from_csv(abspath)
+    except ValueError as e:
+        dbt.exceptions.raise_compiler_error(str(e), node)
+    table.original_abspath = abspath
+    node['agate_table'] = table
+    return node
+
+
+def load_and_parse_seeds(package_name, root_project, all_projects, root_dir,
+                         relative_dirs, resource_type, tags=None, macros=None):
+    extension = "[!.#~]*.csv"
+    if dbt.flags.STRICT_MODE:
+        dbt.contracts.project.validate_list(all_projects)
+    file_matches = dbt.clients.system.find_matching(
+        root_dir,
+        relative_dirs,
+        extension)
+    result = {}
+    for file_match in file_matches:
+        node = parse_seed_file(file_match, root_dir, package_name)
+        node_path = node['unique_id']
+        parsed = parse_node(node, node_path, root_project,
+                            all_projects.get(package_name),
+                            all_projects, tags=tags, macros=macros)
+        # parsed['empty'] = False
+        result[node_path] = parsed
+
+    dbt.contracts.graph.parsed.validate_nodes(result)
+    return result
