@@ -10,7 +10,10 @@ import dbt.contracts.connection
 import dbt.clients.yaml_helper
 import dbt.clients.jinja
 import dbt.compat
-from dbt.logger import GLOBAL_LOGGER as logger
+import dbt.context.common
+import dbt.clients.system
+
+from dbt.logger import GLOBAL_LOGGER as logger  # noqa
 
 default_project_cfg = {
     'source-paths': ['models'],
@@ -23,7 +26,7 @@ default_project_cfg = {
     'target': 'default',
     'models': {},
     'profile': None,
-    'repositories': [],
+    'packages': [],
     'modules-path': 'dbt_modules'
 }
 
@@ -78,11 +81,25 @@ class Project(object):
 
         if self.profile_to_load in self.profiles:
             self.cfg.update(self.profiles[self.profile_to_load])
+            self.compile_and_update_target()
+
         else:
             raise DbtProjectError(
                 "Could not find profile named '{}'"
                 .format(self.profile_to_load), self)
 
+        if self.cfg.get('models') is None:
+            self.cfg['models'] = {}
+
+        if self.cfg['models'].get('vars') is None:
+            self.cfg['models']['vars'] = {}
+
+        global_vars = dbt.utils.parse_cli_vars(getattr(args, 'vars', '{}'))
+        if 'vars' not in self.cfg['models']:
+            self.cfg['models']['vars'] = {}
+
+        self.cfg['models']['vars'].update(global_vars)
+
     def __str__(self):
         return pprint.pformat({'project': self.cfg, 'profiles': self.profiles})
 
@@ -118,8 +135,7 @@ class Project(object):
             is_str = isinstance(value, dbt.compat.basestring)
 
             if is_str:
-                node = "config key: '{}'".format(key)
-                compiled_val = dbt.clients.jinja.get_rendered(value, ctx, node)
+                compiled_val = dbt.clients.jinja.get_rendered(value, ctx)
             else:
                 compiled_val = value
 
@@ -127,6 +143,10 @@ class Project(object):
 
         return compiled
 
+    def compile_and_update_target(self):
+        target = self.cfg['target']
+        self.cfg['outputs'][target].update(self.run_environment())
+
     def run_environment(self):
         target_name = self.cfg['target']
         if target_name in self.cfg['outputs']:
@@ -142,18 +162,9 @@ class Project(object):
         ctx['name'] = self.cfg['target']
         return ctx
 
-    def get_env_var(self, var, default=None):
-        if var in os.environ:
-            return os.environ[var]
-        elif default is not None:
-            return default
-        else:
-            msg = "Env var required but not provided: '{}'".format(var)
-            dbt.clients.jinja.undefined_error(msg)
-
     def base_context(self):
         return {
-            'env_var': self.get_env_var
+            'env_var': dbt.context.common._env_var
         }
 
     def context(self):
@@ -226,20 +237,45 @@ def read_profiles(profiles_dir=None):
 
     raw_profiles = dbt.config.read_profile(profiles_dir)
 
-    profiles = {k: v for (k, v) in raw_profiles.items() if k != 'config'}
+    if raw_profiles is None:
+        profiles = {}
+    else:
+        profiles = {k: v for (k, v) in raw_profiles.items() if k != 'config'}
+
     return profiles
 
 
-def read_project(filename, profiles_dir=None, validate=True,
+def read_packages(project_dir):
+
+    package_filepath = dbt.clients.system.resolve_path_from_base(
+            'packages.yml', project_dir)
+
+    if dbt.clients.system.path_exists(package_filepath):
+        package_file_contents = dbt.clients.system.load_file_contents(
+                package_filepath)
+        package_cfg = dbt.clients.yaml_helper.load_yaml_text(
+                package_file_contents)
+    else:
+        package_cfg = {}
+
+    return package_cfg.get('packages', [])
+
+
+def read_project(project_filepath, profiles_dir=None, validate=True,
                  profile_to_load=None, args=None):
     if profiles_dir is None:
         profiles_dir = default_profiles_dir
 
-    project_file_contents = dbt.clients.system.load_file_contents(filename)
+    project_dir = os.path.dirname(os.path.abspath(project_filepath))
+    project_file_contents = dbt.clients.system.load_file_contents(
+            project_filepath)
 
     project_cfg = dbt.clients.yaml_helper.load_yaml_text(project_file_contents)
-    project_cfg['project-root'] = os.path.dirname(
-        os.path.abspath(filename))
+    package_cfg = read_packages(project_dir)
+
+    project_cfg['project-root'] = project_dir
+    project_cfg['packages'] = package_cfg
+
     profiles = read_profiles(profiles_dir)
     proj = Project(project_cfg,
                    profiles,
