@@ -31,7 +31,7 @@ def get_latest_version():
     try:
         f = urlopen(REMOTE_VERSION_FILE)
         contents = f.read()
-    except:
+    except Exception:
         contents = ''
     if hasattr(contents, 'decode'):
         contents = contents.decode('utf-8')
@@ -42,7 +42,7 @@ def not_latest():
     return """Your version of dbt is out of date! You can find instructions
     for upgrading here:
 
-    https://dbt.readme.io/docs/installation
+    https://docs.getdbt.com/docs/installation
     """
 
 
@@ -67,6 +67,6 @@ def is_latest():
     return installed == latest
 
 
-__version__ = '0.8.3'
+__version__ = '0.10.0'
 installed = get_version()
 latest = get_latest_version()
